/*****************************************************************************
*
* ALPS Project: Algorithms and Libraries for Physics Simulations
*
* ALPS Light Libraries
*
* Copyright (C) 2003 by Matthias Troyer <troyer@comp-phys.org>
*
* This software is part of the "ALPS Light" Libraries, public-domain
* part of the ALPS Libraries. If you need the full functionality of
* the ALPS Libraries, such as Lattice, Model, Scheduler, etc, please
* use the full version of ALPS Libraries, which is available from
* http://alps.comp-phys.org/.
*
* Permission is hereby granted, free of charge, to any person or organization
* obtaining a copy of the software and accompanying documentation covered by
* this license (the "Software") to use, reproduce, display, distribute,
* execute, and transmit the Software, and to prepare derivative works of the
* Software, and to permit third-parties to whom the Software is furnished to
* do so, all subject to the following:
*
* The copyright notices in the Software and this entire statement, including
* the above license grant, this restriction and the following disclaimer,
* must be included in all copies of the Software, in whole or in part, and
* all derivative works of the Software, unless such copies or derivative
* works are solely in the form of machine-executable object code generated by
* a source language processor.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
* FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT 
* SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE 
* FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE, 
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
* DEALINGS IN THE SOFTWARE.
*
*****************************************************************************/

/* $Id: function_objects.hpp,v 1.3 2004/03/16 15:12:30 wistaria Exp $ */

#ifndef FUNCTION_OBJECTS_HPP
#define FUNCTION_OBJECTS_HPP

#include <functional>

namespace boost
{
  // improved function objects taking optionally different argument types
  template <class Arg1, class Arg2=Arg1, class Result=Arg1>
  struct plus : std::binary_function<Arg1, Arg2, Result> {
    Result operator () (const Arg1& x, const Arg2& y) const { return x + y; }
  };

  template <class Arg1, class Arg2=Arg1, class Result=Arg1>
  struct minus : std::binary_function<Arg1, Arg2, Result>  {
    Result operator () (const Arg1& x, const Arg2& y) const { return x - y; }
  };

  template <class Arg1, class Arg2=Arg1, class Result=Arg1>
  struct multiplies : std::binary_function<Arg1, Arg2, Result> {
    Result operator () (const Arg1& x, const Arg2& y) const { return x * y; }
  };

  template <class Arg1, class Arg2=Arg1, class Result=Arg1>
  struct divides : std::binary_function<Arg1, Arg2, Result> {
    Result operator () (const Arg1& x, const Arg2& y) const { return x / y; }
  };

  template <class Arg1, class Arg2=Arg1, class Result=Arg1>
  struct modulus : std::binary_function<Arg1, Arg2, Result> {
    Result operator () (const Arg1& x, const Arg2& y) const { return x % y; }
  };

  template <class Arg1, class Arg2=Arg1, class Result=bool>
  struct logical_and : std::binary_function<Arg1, Arg2, Result> {
    Result operator () (const Arg1& x, const Arg2& y) const { return x && y; }
  };

  template <class Arg1, class Arg2=Arg1, class Result=bool>
  struct logical_or : std::binary_function<Arg1, Arg2, Result> {
    Result operator () (const Arg1& x, const Arg2& y) const { return x || y; }
  };

  // additional function objects for bit operations missing from the standard
  
  template <class Arg1, class Arg2=Arg1, class Result=Arg1>
  struct bit_and : std::binary_function<Arg1, Arg2, Result> {
    Result operator () (const Arg1& x, const Arg2& y) const { return x & y; }
  };

  template <class Arg1, class Arg2=Arg1, class Result=Arg1>
  struct bit_or : std::binary_function<Arg1, Arg2, Result> {
    Result operator () (const Arg1& x, const Arg2& y) const { return x | y; }
  };

  template <class Arg1, class Arg2=Arg1, class Result=Arg1>
  struct bit_xor : std::binary_function<Arg1, Arg2, Result> {
    Result operator () (const Arg1& x, const Arg2& y) const { return x ^ y; }
  };

  template <class T>
  struct bit_not : std::unary_function<T, T> {
    T operator () (const T& x) const { return ~x; }
  };

} // namespace boost

#endif
